//
// MessagePack for Java
//
// Copyright (C) 2009-2010 FURUHASHI Sadayuki
//
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.
//
package org.msgpack.template;

import java.util.List;
import java.util.ArrayList;
import java.io.IOException;
import org.msgpack.*;

public class ListTemplate implements Template {
	static void load() { }

	private Template elementTemplate;

	public ListTemplate(Template elementTemplate) {
		this.elementTemplate = elementTemplate;
	}

	public Template getElementTemplate() {
		return elementTemplate;
	}

	@SuppressWarnings("unchecked")
	public void pack(Packer pk, Object target) throws IOException {
		if (! (target instanceof List)) {
			if (target == null) {
				throw new MessageTypeException(new NullPointerException("target is null."));
			}
			throw new MessageTypeException("target is not List type: " + target.getClass());
		}
		List<Object> list = (List<Object>)target;
		pk.packArray(list.size());
		for(Object element : list) {
			elementTemplate.pack(pk, element);
		}
	}

	@SuppressWarnings("unchecked")
	public Object unpack(Unpacker pac, Object to) throws IOException, MessageTypeException {
		int length = pac.unpackArray();
		List<Object> list;
		if(to == null) {
			list = new ArrayList<Object>(length);
		} else {
			list = (List<Object>) to;
			list.clear();
		}
		for(; length > 0; length--) {
			list.add( elementTemplate.unpack(pac, null) );
		}
		return list;
	}

	@SuppressWarnings("unchecked")
	public Object convert(MessagePackObject from, Object to) throws MessageTypeException {
		MessagePackObject[] array = from.asArray();
		List<Object> list;
		if(to == null) {
			list = new ArrayList<Object>(array.length);
		} else {
			// TODO: optimize if list is instanceof ArrayList
			list = (List<Object>) to;
			list.clear();
		}
		for(MessagePackObject element : array) {
			list.add( elementTemplate.convert(element, null) );
		}
		return list;
	}

	static {
		TemplateRegistry.registerGeneric(List.class, new GenericTemplate1(ListTemplate.class));
		TemplateRegistry.register(List.class, new ListTemplate(AnyTemplate.getInstance()));
	}
}

