//
// MessagePack for Java
//
// Copyright (C) 2009-2010 FURUHASHI Sadayuki
//
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.
//
package org.msgpack.template;

import java.io.IOException;
import org.msgpack.*;

public class LongTemplate implements Template {
	private LongTemplate() { }

	public void pack(Packer pk, Object target) throws IOException {
		try {
			pk.packLong((Long)target);
		} catch (NullPointerException e) {
			throw new MessageTypeException("target is null.", e);
		}
	}

	public Object unpack(Unpacker pac, Object to) throws IOException, MessageTypeException {
		return pac.unpackLong();
	}

	public Object convert(MessagePackObject from, Object to) throws MessageTypeException {
		return from.asLong();
	}

	static public LongTemplate getInstance() {
		return instance;
	}

	static final LongTemplate instance = new LongTemplate();

	static {
		TemplateRegistry.register(Long.class, instance);
		TemplateRegistry.register(long.class, instance);
	}
}

